import os

import pytest
from dotenv import load_dotenv
from langchain_openai import ChatOpenAI

from langchain_memgraph.chains.graph_qa import MemgraphQAChain
from langchain_memgraph.graphs.memgraph import MemgraphLangChain


# Load environment variables from .env
load_dotenv()


@pytest.fixture(scope="module")
def memgraph_connection():
    """Setup Memgraph connection fixture."""
    url = os.getenv("MEMGRAPH_URL", "bolt://localhost:7687")
    username = os.getenv("MEMGRAPH_USER", "")
    password = os.getenv("MEMGRAPH_PASSWORD", "")

    graph = MemgraphLangChain(url=url, username=username, password=password)
    yield graph

    # Cleanup: clear the database after test
    graph.query("MATCH (n) DETACH DELETE n")


@pytest.fixture(scope="module")
def memgraph_chain(memgraph_connection):
    """Set up MemgraphQAChain with OpenAI LLM."""
    os.environ["OPENAI_API_KEY"] = os.getenv("OPENAI_API_KEY", "")

    chain = MemgraphQAChain.from_llm(
        ChatOpenAI(temperature=0),
        graph=memgraph_connection,
        model_name="gpt-4-turbo",
        allow_dangerous_requests=True,
    )
    return chain


def test_seed_graph(memgraph_connection):
    """Test to seed the graph with Baldur's Gate 3 data."""
    query = """
        MERGE (g:Game {name: "Baldur\'s Gate 3"})
        WITH g, ["PlayStation 5", "Mac OS", "Windows", "Xbox Series X/S"] AS platforms,
                ["Adventure", "Role-Playing Game", "Strategy"] AS genres
        FOREACH (platform IN platforms |
            MERGE (p:Platform {name: platform})
            MERGE (g)-[:AVAILABLE_ON]->(p)
        )
        FOREACH (genre IN genres |
            MERGE (gn:Genre {name: genre})
            MERGE (g)-[:HAS_GENRE]->(gn)
        )
        MERGE (p:Publisher {name: "Larian Studios"})
        MERGE (g)-[:PUBLISHED_BY]->(p);
    """
    memgraph_connection.query(query)
    memgraph_connection.refresh_schema()

    # Verify nodes exist
    game_exists = memgraph_connection.query(
        """MATCH (g:Game {name: "Baldur's Gate 3"}) RETURN g"""
    )
    assert len(game_exists) > 0, "Game node not created!"

    print("✅ Graph seeded successfully")


def test_qa_chain(memgraph_chain):
    """Test QA chain to verify platform data retrieval."""
    response = memgraph_chain.invoke("Which platforms is Baldur's Gate 3 available on?")
    result = response["result"].lower()

    # Assert the response contains all expected platforms
    expected_platforms = ["playstation 5", "mac os", "windows", "xbox series x/s"]

    for platform in expected_platforms:
        assert platform in result, f"Platform '{platform}' not found in response!"

    print("✅ QA Chain returned expected platforms")



from typing import Type

from langchain_tests.integration_tests import ToolsIntegrationTests
from langchain_memgraph.tools import (
    RunQueryTool,
    RunShowSchemaInfoTool,
    RunPageRankMemgraphTool,
    RunShowStorageInfoTool,
    RunShowConstraintInfoTool,
    RunShowIndexInfoTool,
    RunShowConfigTool,
    RunShowTriggersTool,
    RunBetweennessCentralityTool,
    RunNodeNeighborhoodTool,
    RunNodeVectorSearchTool,
)
from memgraph_toolbox.api.memgraph import Memgraph


class TestSchemaInfoIntegration(ToolsIntegrationTests):
    @property
    def tool_constructor(self) -> Type[RunShowSchemaInfoTool]:
        return RunShowSchemaInfoTool

    @property
    def tool_constructor_params(self) -> dict:
        return {"db": Memgraph("bolt://localhost:7687", "", "")}

    @property
    def tool_invoke_params_example(self) -> dict:
        """
        Returns empty dict since ShowSchemaInfoTool doesn't require any parameters
        """
        return {}


class TestCypherIntegration(ToolsIntegrationTests):
    @property
    def tool_constructor(self) -> Type[RunQueryTool]:
        return RunQueryTool

    @property
    def tool_constructor_params(self) -> dict:
        return {"db": Memgraph("bolt://localhost:7687", "", "")}

    @property
    def tool_invoke_params_example(self) -> dict:
        return {"query": "MATCH (n) RETURN n LIMIT 1"}


class TestPageRankIntegration(ToolsIntegrationTests):
    @property
    def tool_constructor(self) -> Type[RunPageRankMemgraphTool]:
        return RunPageRankMemgraphTool

    @property
    def tool_constructor_params(self) -> dict:
        return {"db": Memgraph("bolt://localhost:7687", "", "")}

    @property
    def tool_invoke_params_example(self) -> dict:
        return {"limit": 5}


class TestStorageInfoIntegration(ToolsIntegrationTests):
    @property
    def tool_constructor(self) -> Type[RunShowStorageInfoTool]:
        return RunShowStorageInfoTool

    @property
    def tool_constructor_params(self) -> dict:
        return {"db": Memgraph("bolt://localhost:7687", "", "")}

    @property
    def tool_invoke_params_example(self) -> dict:
        """
        Returns empty dict since ShowStorageInfoTool doesn't require any parameters.
        """
        return {}


class TestConstraintInfoIntegration(ToolsIntegrationTests):
    @property
    def tool_constructor(self) -> Type[RunShowConstraintInfoTool]:
        return RunShowConstraintInfoTool

    @property
    def tool_constructor_params(self) -> dict:
        return {"db": Memgraph("bolt://localhost:7687", "", "")}

    @property
    def tool_invoke_params_example(self) -> dict:
        """
        Returns empty dict since ShowConstraintInfoTool doesn't require any parameters.
        """
        return {}


class TestIndexInfoIntegration(ToolsIntegrationTests):
    @property
    def tool_constructor(self) -> Type[RunShowIndexInfoTool]:
        return RunShowIndexInfoTool

    @property
    def tool_constructor_params(self) -> dict:
        return {"db": Memgraph("bolt://localhost:7687", "", "")}

    @property
    def tool_invoke_params_example(self) -> dict:
        """
        Returns empty dict since ShowIndexInfoTool doesn't require any parameters.
        """
        return {}


class TestConfigInfoIntegration(ToolsIntegrationTests):
    @property
    def tool_constructor(self) -> Type[RunShowConfigTool]:
        return RunShowConfigTool

    @property
    def tool_constructor_params(self) -> dict:
        return {"db": Memgraph("bolt://localhost:7687", "", "")}

    @property
    def tool_invoke_params_example(self) -> dict:
        """
        Returns empty dict since ShowConfigTool doesn't require any parameters.
        """
        return {}


class TestTriggersIntegration(ToolsIntegrationTests):
    @property
    def tool_constructor(self) -> Type[RunShowTriggersTool]:
        return RunShowTriggersTool

    @property
    def tool_constructor_params(self) -> dict:
        return {"db": Memgraph("bolt://localhost:7687", "", "")}

    @property
    def tool_invoke_params_example(self) -> dict:
        """
        Returns empty dict since ShowTriggersTool doesn't require any parameters.
        """
        return {}


class TestBetweennessCentralityIntegration(ToolsIntegrationTests):
    @property
    def tool_constructor(self) -> Type[RunBetweennessCentralityTool]:
        return RunBetweennessCentralityTool

    @property
    def tool_constructor_params(self) -> dict:
        return {"db": Memgraph("bolt://localhost:7687", "", "")}

    @property
    def tool_invoke_params_example(self) -> dict:
        return {"isDirectionIgnored": True, "limit": 5}


class TestNodeNeighborhoodIntegration(ToolsIntegrationTests):
    @property
    def tool_constructor(self) -> Type[RunNodeNeighborhoodTool]:
        return RunNodeNeighborhoodTool

    @property
    def tool_constructor_params(self) -> dict:
        return {"db": Memgraph("bolt://localhost:7687", "", "")}

    @property
    def tool_invoke_params_example(self) -> dict:
        return {"node_id": "1", "max_distance": 2, "limit": 10}


class TestNodeVectorSearchIntegration(ToolsIntegrationTests):
    @property
    def tool_constructor(self) -> Type[RunNodeVectorSearchTool]:
        return RunNodeVectorSearchTool

    @property
    def tool_constructor_params(self) -> dict:
        return {"db": Memgraph("bolt://localhost:7687", "", "")}

    @property
    def tool_invoke_params_example(self) -> dict:
        return {"index_name": "test_index", "query_vector": [1.0, 2.0, 3.0], "limit": 5}



import os
from getpass import getpass

import pytest
from dotenv import load_dotenv
from langchain.chat_models import init_chat_model
from langgraph.prebuilt import create_react_agent

from langchain_memgraph import MemgraphToolkit
from memgraph_toolbox.api.memgraph import Memgraph

# Load environment variables
load_dotenv()


@pytest.fixture(scope="module")
def memgraph_connection():
    """Setup Memgraph connection fixture."""
    url = os.getenv("MEMGRAPH_URL", "bolt://localhost:7687")
    username = os.getenv("MEMGRAPH_USER", "")
    password = os.getenv("MEMGRAPH_PASSWORD", "")

    graph = Memgraph(url=url, username=username, password=password)
    yield graph

    # Cleanup: clear the database after test
    graph.query("MATCH (n) DETACH DELETE n")


@pytest.fixture(scope="module")
def memgraph_agent():
    url = os.getenv("MEMGRAPH_URL", "bolt://localhost:7687")
    username = os.getenv("MEMGRAPH_USER", "")
    password = os.getenv("MEMGRAPH_PASSWORD", "")

    """Set up Memgraph agent with React pattern."""
    if not os.environ.get("OPENAI_API_KEY"):
        os.environ["OPENAI_API_KEY"] = getpass("Enter API key for OpenAI: ")

    llm = init_chat_model("gpt-4o-mini", model_provider="openai")

    db = Memgraph(url=url, username=username, password=password)
    toolkit = MemgraphToolkit(db=db, llm=llm)

    agent_executor = create_react_agent(
        llm,
        toolkit.get_tools(),
        prompt="You will get a cypher query, try to execute it on the Memgraph database.",
    )
    return agent_executor


def test_seed_graph(memgraph_connection):
    """Test to seed the graph with Jon Snow node"""
    query = """
       CREATE (c:Character {name: 'Jon Snow', house: 'Stark', title: 'King in the North'})
    """
    memgraph_connection.query(query)

    # Verify nodes exist
    game_exists = memgraph_connection.query(
        """MATCH (c:Character {name: 'Jon Snow'}) RETURN c"""
    )
    assert len(game_exists) > 0, "Game node not created!"


def test_memgraph_agent(memgraph_agent):
    """Test Memgraph agent executes a Cypher query correctly."""
    example_query = "MATCH (n) WHERE n.name = 'Jon Snow' RETURN n"
    events = memgraph_agent.stream(
        {"messages": [("user", example_query)]},
        stream_mode="values",
    )

    last_event = None
    for event in events:
        last_event = event
        event["messages"][-1].pretty_print()

    assert last_event, "Agent did not return any results!"
    assert "Jon Snow" in str(
        last_event["messages"][-1]
    ), "Expected 'Jon Snow' in the final result!"
